<?php
/* $Id: antiHotLinks.php 71 2011-01-16 16:09:01Z sarterm $ */
define("FORBID", 0);
define("REDIRECT", 1);
define("HANDLER", 2);
define("ALLOW", 3);
define("LOCALREDIRECT", 4);

include_once(dirname(__FILE__)."/config.inc.php");

function allow($file) {
    if (defined("ROOTPATH")
        && ROOTPATH != "") {
        $path = ROOTPATH."/$file";
    }
    else {
        $path = $file;
    }
    if (file_exists($path)) {
        $mime_type = mime_content_type($path);
        header("Content-Type: $mime_type");
        readfile($path);
    }
    else {
        if (defined("ERROR404")
            && ERROR404 != ""
            && file_exists(dirname(__FILE__)."/".ERROR404)) {
            require(dirname(__FILE__)."/".ERROR404);
        }
        else {
            header("HTTP/1.1 404 Not Found");
        }
    }
    die();
}

function forbid() {
    if (defined("ERROR403") != ""
        && ERROR403 != ""
        && file_exists(dirname(__FILE__)."/".ERROR403)) {
        require(dirname(__FILE__)."/".ERROR403);
    }
    else {
        header("HTTP/1.1 403 Forbidden");
    }
    die();
}

function redirect($file, $rule) {
    if (isset($rule["pattern"])
        && isset($rule["replacement"])) {
        $redirectTo = preg_replace($rule["pattern"], $rule["replacement"], $file);
    }
    else if (isset($rule["replacement"])) {
        $redirectTo = $rule["replacement"];
    }
    else {
        return;
    }
    if (isset($rule["httpCode"])
        && $rule["httpCode"] == 301) {
        header("HTTP/1.1 301 Moved Permanently");
    }
    else {
        header("HTTP/1.1 302 Moved Temporarily");
    }
    header("Location: $redirectTo");
    die();
}

function localredirect($file, $rule) {
    if (isset($rule["pattern"])
        && isset($rule["replacement"])) {
        $redirectTo = preg_replace($rule["pattern"], $rule["replacement"], $file);
    }
    else if (isset($rule["replacement"])) {
        $redirectTo = $rule["replacement"];
    }
    else {
        return;
    }
    if (isset($rule["mime"])) {
        header("Content-Type: ", $rule["mime"]);
    }
    else {
        header("Content-Type: ", mime_content_type($redirectTo));
    }
    readfile($redirectTo);
    die();
}

function callHandler($file, $rule) {
    if (!isset($rule["file"])
        || !file_exists(dirname(__FILE__)."/".$rule["file"])) {
        return;
    }
    else {
        require_once(dirname(__FILE__)."/".$rule["file"]);
        if (!function_exists($rule["func"])) {
            return;
        }
        $rule["func"]($file);
    }
    if (isset($rule["redirectRule"])) {
        redirect($file, $rule["redirectRule"]);
    }
}

function runRule($askedFile, $rule) {
    switch($rule["action"]) {
        case ALLOW:
            allow($askedFile);
            break;
        case REDIRECT:
            if (!isset($rule["redirectRule"])) {
                break;
            }
            redirect($askedFile, $rule["redirectRule"]);
            break;
        case LOCALREDIRECT:
            if (!isset($rule["redirectRule"])) {
                break;
            }
            localredirect($askedFile, $rule["redirectRule"]);
            break;
        case HANDLER;
            if (!isset($rule["handlerRule"])) {
                break;
            }
            callHandler($askedFile, $rule["handlerRule"]);
            break;
        default:
            break;
    }
}

if (!defined("IBDLIBS")) {
    define("IBDLIBS", dirname(__FILE__)."/libs/");
}

$askedFile = isset($_GET["askedFile"]) ? $_GET["askedFile"] : "";
if ($askedFile == ""
    || !isset($AHLRules)
    || !is_array($AHLRules)) {
    forbid();
}
$referer = isset($_SERVER["HTTP_REFERER"]) ? $_SERVER["HTTP_REFERER"] : "";
$agent = isset($_SERVER["HTTP_USER_AGENT"]) ? $_SERVER["HTTP_USER_AGENT"] : "";
$nbRules = sizeof($AHLRules);
for ($i = 0; $i < $nbRules; $i++) {
    $currentRule = $AHLRules[$i];
    if ((isset($currentRule["patternFile"]) && preg_match($currentRule["patternFile"], $askedFile))
        || (isset($currentRule["patternAgent"]) && preg_match($currentRule["patternAgent"], $agent))
        || (isset($currentRule["patternReferer"]) && preg_match($currentRule["patternReferer"], $referer))) {
        if (isset($currentRule["ifBot"])) {
            require_once(IBDLIBS."IBDInfosNavigateur/navigateur.cls.php");
            $infosNav = new IBDNavigateur(true, true);
            if (isset($infosNav->navigateur["Crawler"]) && $infosNav->navigateur["Crawler"]) {
                runRule($askedFile, $currentRule["ifBot"]);
            }
        }
        if (isset($currentRule["ifImageSearch"])
            || isset($currentRule["ifSearch"])) {
            require_once(IBDLIBS."IBDInfosReferant/referant.cls.php");
            $infosRef = new IBDReferant(true, true);
            if ($infosRef->estMoteur) {
                if (isset($currentRule["ifImageSearch"])
                    && $infosRef->varianteMoteur == "Images") {
                    runRule($askedFile, $currentRule["ifImageSearch"]);
                }
                else if (isset($currentRule["ifSearch"])) {
                    runRule($askedFile, $currentRule["ifSearch"]);
                }
            }
        }
        if (isset($currentRule["defaultRule"])) {
            runRule($askedFile, $currentRule["defaultRule"]);
        }
    }
}

forbid();
?>