<?php
/* $Id: navigateur.cls.php 73 2011-01-16 16:09:15Z sarterm $ */
include_once(dirname(__FILE__)."/config.inc.php");

/*
 * Stocke les informations sur un navigateur.
 * - agent : le user-agent original,
 * - agentMin : le user-argent en minuscules,
 * - navigateur : tableau contenant les informations sur le navigateur, dans un
 *   format similaire à celui de la fonction get_browser() de PHP. Les noms des
 *   champs commencent par une majuscule.
 */
class IBDNavigateur {
    public $agent                = "";
    public $navigateur           = false;
    private $agentMin            = "";
    private $baseCache           = false;
    private $baseBC              = Array();
    private $baseFormat          = Array();
    private $init                = false;
    private $lite_baseCache      = false;
    private $lite_baseBC         = Array();
    private $lite_baseFormatLite = Array();
    private $lite_init           = false;
    private $formattage          = Array(
    	"\\" => "\\\\",
    	"." => "\\.",
    	"?" => ".",
    	"*" => ".*",
    	"^" => "\\^",
    	"[" => "\\[",
    	"]" => "\\]",
    	"|" => "\\|",
    	"(" => "\\(",
    	")" => "\\)",
    	"+" => "\\+",
    	"{" => "\\{",
    	"}" => "\\}",
    	'$' => '\\$',
    	" " => "\s*");

    /*
     * Constructeur de la classe.
     * Le constructeur prend une chaîne user-agent en paramètre et effectue la
     * reconnaissance des informations détaillées.
     * 
     * @param agent La chaîne user-agent du navigateur. Si ce paramètre vaut
     *              true, la chaîne est récupérée dans la super globale
     *              $_SERVER. S'il vaut false, l'analyse est ignorée.
     * @param fullBase Si à true, utilisation de la base php_browscap.ini.
     *                 Sinon, utilisation de la base liste_php_browscap.ini.
     * @param initOnly Si à true, l'object est initialisé, mais sans analyse de
     *                   l'agent passé en paramètre.
     * @return La durée d'exécution, en secondes.
     */
    function IBDNavigateur($agent = true, $fullBase = true) {
        global $cfgIBDIN;
        $pref = "";
        $now = explode(' ',microtime());
        $start = $now[0] + $now[1];
        
        // Récupération de l'agent courant dans $_SERVER
        if ($agent === true) {
            $agent = isset($_SERVER["HTTP_USER_AGENT"]) ? $_SERVER["HTTP_USER_AGENT"] : "";   
            if ($agent == "") {
                return;
            }
        }
        
        $this->agent = $agent;
        $this->agentMin = strtolower($agent);
        $this->navigateur = false;
        
        if (!$fullBase) {
            $pref = "lite_";
        }
        if (!$this->{$pref."init"}) {
            $this->init($pref);
        }
        
        if ($agent === false) {
            return;
        }
        
        // Recherche l'agent dans le cache
        if ($cfgIBDIN["useCacheAgents"]) {
            $this->navigateur = $this->extraireCache($agent, $pref);
            if ($this->navigateur !== false) {
                $now = explode(' ',microtime());
                $end = $now[0] + $now[1];
                return $end - $start;
            }
        }
        
        $this->navigateur = Array();
        
        // Analyse de l'agent
        foreach($this->{$pref."baseFormat"} as $ereg => $traduction)
        {
            $valeur = $this->{$pref."baseBC"}[$traduction];
            if (!array_key_exists("Parent", $valeur)) {
                continue;
            }
            if (preg_match("#".$ereg."#", $this->agentMin))
            {
                $this->navigateur = Array("Agent" => $agent, "browser_name_regex" => $ereg, "browser_name_patern" => $traduction) + $valeur;
                while (array_key_exists("Parent", $valeur) && array_key_exists($valeur["Parent"], $this->{$pref."baseBC"})) {
                    $parent = $valeur["Parent"];
                    $valeur = $this->{$pref."baseBC"}[$parent];
                    $this->navigateur += $valeur;
                }
                $this->navigateur["browser_name_regex"] = str_replace("\s*", " ", $this->navigateur["browser_name_regex"]);
                break; /* for sur baseFormat */
            }
        }
        
        // Sauvegarde du résultat dans le cache
        if ($cfgIBDIN["useCacheAgents"]) {
            $this->ajouterCache($agent, $this->navigateur, $pref);
        }
        
        $now = explode(' ',microtime());
        $end = $now[0] + $now[1];
        return $end - $start;
    }
        
    /*
     * Initialise baseBC et baseFormat à partir de fichier php_browscap.ini ou
     * à partir du cache.
     */
    private function init($pref = "") {
        global $cfgIBDIN;
        clearstatcache();
        $cacheFileBC     = dirname(__FILE__)."/".$pref."baseBC.dat";
        $cacheFileFormat = dirname(__FILE__)."/".$pref."baseFormat.dat";
        $cacheFileUA     = dirname(__FILE__)."/".$pref."baseCache.dat";
        $browscapFile    = dirname(__FILE__)."/".$pref."php_browscap.ini";
        $browscapDate    = filemtime($browscapFile);
        if (file_exists($cacheFileUA)
            && $browscapDate > filemtime($cacheFileUA)) {
            unlink($cacheFileUA);
        }
        if ($cfgIBDIN["useCache"]
            && file_exists($cacheFileBC)
            && $browscapDate < filemtime($cacheFileBC)) {
            $this->{$pref."baseBC"} = unserialize(file_get_contents($cacheFileBC));
        }
        else {
            $this->{$pref."baseBC"} = parse_ini_file($browscapFile, true);
            if ($cfgIBDIN["useCache"]) {
                file_put_contents($cacheFileBC, serialize($this->{$pref."baseBC"}));
            }
        }
        if ($cfgIBDIN["useCache"]
            && file_exists($cacheFileFormat)
            && $browscapDate < filemtime($cacheFileFormat)) {
            $this->{$pref."baseFormat"} = unserialize(file_get_contents($cacheFileFormat));
        }
        else {
            foreach ($this->{$pref."baseBC"} as $cle => $valeur) {
                $this->{$pref."baseFormat"}[$this->formateEreg($cle)] = $cle;
            }
            if ($cfgIBDIN["useCache"]) {
                file_put_contents($cacheFileFormat, serialize($this->{$pref."baseFormat"}));
            }
        }
        $this->{$pref."init"} = true;
    }
        
    /*
     * Converti les expressions régulières du format php_browscap.ini en un
     * format exploitable par preg_match.
	 *
	 * @param ereg L'expression régulière à convertir.
	 * @return L'expression régulière convertie.
     */
    private function formateEreg($ereg) {
        $ereg = strtolower($ereg);
        reset($this->formattage);
        foreach ($this->formattage as $cle => $valeur) {
            $ereg = str_replace($cle, $valeur, $ereg);
        }
        return "^".$ereg."$";
    }
    
    /*
     * Vide le cache des agents déjà traités.
     * 
     * @param pref Le préfixe de la base à nettoyer (vide ou "lite_").
     */
    public function viderCache($pref = "") {
        $cacheFileUA = dirname(__FILE__)."/".$pref."baseCache.dat";
        $this->{$pref."baseCache"} = Array();
        if (file_exists($cacheFileUA)) {
            unlink($cacheFileUA);
        }
        return false;
    }
    
    /*
     * Cherche un agent dans le cache des agents déjà traités.
     * 
     * @param agent La chaîne user-agent à chercher.
     * @param pref  Le préfixe de la base à nettoyer (vide ou "lite_").
	 * @return Le tableau décrivant le navigateur si l'agent a été trouvé dans
	 *         le cache. false sinon.
     */
    private function extraireCache($agent, $pref = "") {
        $cacheFileUA = dirname(__FILE__)."/".$pref."baseCache.dat";
        if ($this->{$pref."baseCache"} === false) {
            if (file_exists($cacheFileUA)) {
                $this->{$pref."baseCache"} = unserialize(file_get_contents($cacheFileUA));
            }
        }
        if (isset($this->{$pref."baseCache"}[$agent])) {
            return $this->{$pref."baseCache"}[$agent];
        }
        return false;
    }
    
    /*
     * Ajoute un agent dans le cache.
     * 
     * @param agent La chaîne user-agent à ajouter.
     * @param pref Le préfixe de la base à nettoyer (vide ou "lite_").
	 * @param navigateur Le tableau décrivant le navigateur correspondant.
     */
    private function ajouterCache($agent, $navigateur, $pref = "") {
        global $cfgIBDIN;
        $cacheFileUA = dirname(__FILE__)."/".$pref."baseCache.dat";
        if ($this->{$pref."baseCache"} === false) {
            $this->{$pref."baseCache"} = Array();
            if (file_exists($cacheFileUA)) {
                $this->{$pref."baseCache"} = unserialize(file_get_contents($cacheFileUA));
            }
        }
        if (!isset($this->{$pref."baseCache"}[$agent])) {
            $this->{$pref."baseCache"}[$agent] = $navigateur;
            file_put_contents($cacheFileUA, serialize($this->{$pref."baseCache"}));
            clearstatcache();
            /* Si la taille du cache atteint le max, on supprime la moitié des éléments */
            while (filesize($cacheFileUA) > $cfgIBDIN["maxCache"]) {
                $tailleCache = sizeof($this->{$pref."baseCache"});
                $cles = array_keys($this->{$pref."baseCache"});
                for ($i = 0; $i < ($tailleCache / 2) ; $i++) {
                    unset($this->{$pref."baseCache"}[$cles[$i]]);
                }
                file_put_contents($cacheFileUA, serialize($this->{$pref."baseCache"}));
                clearstatcache();
            }
        }
    }
    
    /*
     * Renvoie un tableau décrivant le navigateur dans un format identique à
     * celui renvoyé par la fonction get_browser de PHP.
     *
     * @return Un tableau contenant les mêmes champs qu'un tableau renvoyé par
     *         get_browser().
     */
    public function get_browser() {
        if ($this->navigateur === false) {
            return Array();
        }
        $ret = array_change_key_case($this->navigateur);
        unset($ret["agent"]);
        return $ret;
    }
}
?>